==> neper-1.8.0.tgz: neper-1.8.0/src/Utils/TessUtils/Tools/TessTools1.c <==
/* This file is part of the 'neper' program. */
/* Copyright (C) 2004-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "TessTools.h"

/* ************************************************************************
 * Recording of the edge lengths from the edge properties.
 */
void
IniTessEdgeLength (struct TESS *pTess)
{
  int i, j;			/* mute variables */
  int numa, numb;		/* number of the first and second vetices */
  double *cooa;			/* tmp variable: coordinates of the */
  double *coob;			/* first and second vertices */

  cooa = oneDDoubleDAlloc (3);
  coob = oneDDoubleDAlloc (3);

  (*pTess).EdgeLength = oneDDoubleDAlloc ((*pTess).EdgeQty + 1);

  /* For every edge, we temporary record its two vertices and their
   * coordinates. Then, its length is computed and recorded.
   */
  for (i = 1; i <= (*pTess).EdgeQty; i++)
  {
    numa = (*pTess).EdgeVerNb[i][0];
    numb = (*pTess).EdgeVerNb[i][1];

    for (j = 0; j <= 2; j++)
    {
      cooa[j] = (*pTess).VerCoo[numa][j];
      coob[j] = (*pTess).VerCoo[numb][j];
    }

    (*pTess).EdgeLength[i] = threeDDoubleDist (cooa, coob, 0);
  }

  oneDDoubleFree (cooa);
  oneDDoubleFree (coob);

  return;
}

void
SearchDomainFace (struct TESS Tess, int **Face)
{
  int i, j;
  int face;

  for (face = 1; face <= 6; face++)
    Face[face][0] = 0;

  for (i = 1; i <= Tess.FaceQty; i++)
  {
    for (j = 0; j <= 1; j++)
      if (Tess.FaceGerm[i][j] < 0)
      {
	face = -Tess.FaceGerm[i][j];

	Face[face][0]++;
	Face[face][Face[face][0]] = i;

	continue;
      }
  }

  return;
}

void
SearchDomainFaceVer (struct TESS Tess, int **Ver)
{
  int i, j, k;
  int face;
  int **DFace = twoDIntDAlloc (7, Tess.FaceQty + 1);
  int *FaceVer;

  SearchDomainFace (Tess, DFace);

  for (i = 1; i <= 6; i++)
  {
    FaceVer = oneDDoubleDAlloc_int (DFace[i][0] * 50);

    for (j = 1; j <= DFace[i][0]; j++)
    {
      face = DFace[i][j];
      for (k = 1; k <= Tess.FaceVerQty[face]; k++)
      {
	FaceVer[0]++;
	FaceVer[FaceVer[0]] = Tess.FaceVerNb[face][k];
      }
    }

    FaceVer[0] = oneDIntSortNCompress (FaceVer, 1, FaceVer[0]);

    oneDIntCpy (FaceVer, 0, FaceVer[0], Ver[i], 0, 0);

    oneDIntFree (FaceVer);
  }

  twoDIntFree (DFace, 7);

  return;
}


/* Searching the true polyhedra */
void
SearchTruePoly (struct TESS Tess, int **pTruePoly)
{
  int i, j, k, l;
  int ver;
  double ctov, vtob, vtobmin;
  double *coo = oneDDoubleDAlloc (3);

  for (i = 1; i <= Tess.PolyQty; i++)
    for (j = 1; j <= Tess.PolyVerQty[i]; j++)
    {
      ver = Tess.PolyVerNb[i][j];

      ctov = threeDDoubleDist (Tess.CenterCoo[i], Tess.VerCoo[ver], 0);
      vtobmin = 1;

      for (l = 0; l <= 2; l++)
      {
	for (k = 0; k <= 2; k++)
	  coo[k] = Tess.VerCoo[ver][k];

	coo[l] = 0;
	vtob = threeDDoubleDist (Tess.VerCoo[ver], coo, 0);
	if (vtobmin > vtob)
	  vtobmin = vtob;

	coo[l] = 1;
	vtob = threeDDoubleDist (Tess.VerCoo[ver], coo, 0);
	if (vtobmin > vtob)
	  vtobmin = vtob;
      }

      if (vtobmin > ctov)
      {
	(*pTruePoly)[i] = 1;
	break;
      }
    }

  oneDDoubleFree (coo);

  return;
}

/* Searching the true vertices */
void
SearchTrueVer (struct TESS Tess, int *TruePoly, int **pTrueVer)
{
  int i, j;
  int poly;

  for (i = 1; i <= Tess.VerQty; i++)
    for (j = 0; j <= 3; j++)
    {
      poly = Tess.VerGerm[i][j];
      if (poly > 0)
	if (TruePoly[poly] == 1)
	  (*pTrueVer)[i]++;
    }

  return;
}

==> neper-1.8.1.tgz: neper-1.8.1/src/Utils/TessUtils/Tools/TessTools1.c <==
/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2009, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "TessTools.h"

/* ************************************************************************
 * Recording of the edge lengths from the edge properties.
 */
void
IniTessEdgeLength (struct TESS *pTess)
{
  int i, j;			/* mute variables */
  int numa, numb;		/* number of the first and second vetices */
  double *cooa;			/* tmp variable: coordinates of the */
  double *coob;			/* first and second vertices */

  cooa = oneDDoubleDAlloc (3);
  coob = oneDDoubleDAlloc (3);

  (*pTess).EdgeLength = oneDDoubleDAlloc ((*pTess).EdgeQty + 1);

  /* For every edge, we temporary record its two vertices and their
   * coordinates. Then, its length is computed and recorded.
   */
  for (i = 1; i <= (*pTess).EdgeQty; i++)
  {
    numa = (*pTess).EdgeVerNb[i][0];
    numb = (*pTess).EdgeVerNb[i][1];

    for (j = 0; j <= 2; j++)
    {
      cooa[j] = (*pTess).VerCoo[numa][j];
      coob[j] = (*pTess).VerCoo[numb][j];
    }

    (*pTess).EdgeLength[i] = threeDDoubleDist (cooa, coob, 0);
  }

  oneDDoubleFree (cooa);
  oneDDoubleFree (coob);

  return;
}

void
SearchDomainFace (struct TESS Tess, int **Face)
{
  int i, j;
  int face;

  for (face = 1; face <= 6; face++)
    Face[face][0] = 0;

  for (i = 1; i <= Tess.FaceQty; i++)
  {
    for (j = 0; j <= 1; j++)
      if (Tess.FaceGerm[i][j] < 0)
      {
	face = -Tess.FaceGerm[i][j];

	Face[face][0]++;
	Face[face][Face[face][0]] = i;

	continue;
      }
  }

  return;
}

void
SearchDomainFaceVer (struct TESS Tess, int **Ver)
{
  int i, j, k;
  int face;
  int **DFace = twoDIntDAlloc (7, Tess.FaceQty + 1);
  int *FaceVer;

  SearchDomainFace (Tess, DFace);

  for (i = 1; i <= 6; i++)
  {
    FaceVer = ut_alloc_1d_int (DFace[i][0] * 50);

    for (j = 1; j <= DFace[i][0]; j++)
    {
      face = DFace[i][j];
      for (k = 1; k <= Tess.FaceVerQty[face]; k++)
      {
	FaceVer[0]++;
	FaceVer[FaceVer[0]] = Tess.FaceVerNb[face][k];
      }
    }

    FaceVer[0] = oneDIntSortNCompress (FaceVer, 1, FaceVer[0]);

    oneDIntCpy (FaceVer, 0, FaceVer[0], Ver[i], 0, 0);

    oneDIntFree (FaceVer);
  }

  twoDIntFree (DFace, 7);

  return;
}


/* Searching the true polyhedra */
void
SearchTruePoly (struct TESS Tess, int **pTruePoly)
{
  int i, j, k, l;
  int ver;
  double ctov, vtob, vtobmin;
  double *coo = oneDDoubleDAlloc (3);

  for (i = 1; i <= Tess.PolyQty; i++)
    for (j = 1; j <= Tess.PolyVerQty[i]; j++)
    {
      ver = Tess.PolyVerNb[i][j];

      ctov = threeDDoubleDist (Tess.CenterCoo[i], Tess.VerCoo[ver], 0);
      vtobmin = 1;

      for (l = 0; l <= 2; l++)
      {
	for (k = 0; k <= 2; k++)
	  coo[k] = Tess.VerCoo[ver][k];

	coo[l] = 0;
	vtob = threeDDoubleDist (Tess.VerCoo[ver], coo, 0);
	if (vtobmin > vtob)
	  vtobmin = vtob;

	coo[l] = 1;
	vtob = threeDDoubleDist (Tess.VerCoo[ver], coo, 0);
	if (vtobmin > vtob)
	  vtobmin = vtob;
      }

      if (vtobmin > ctov)
      {
	(*pTruePoly)[i] = 1;
	break;
      }
    }

  oneDDoubleFree (coo);

  return;
}

/* Searching the true vertices */
void
SearchTrueVer (struct TESS Tess, int *TruePoly, int **pTrueVer)
{
  int i, j;
  int poly;

  for (i = 1; i <= Tess.VerQty; i++)
    for (j = 0; j <= 3; j++)
    {
      poly = Tess.VerGerm[i][j];
      if (poly > 0)
	if (TruePoly[poly] == 1)
	  (*pTrueVer)[i]++;
    }

  return;
}
